@@ -4,14 +4,12 @@ import (
 	"fmt"
 	"os"
 	"path"
-	"regexp"
 	"runtime"
 	"strings"
 	"testing"
 
 	"github.com/alecthomas/chroma/v2"
 	"github.com/alecthomas/chroma/v2/formatters"
-	"github.com/alecthomas/chroma/v2/lexers"
 	"github.com/alecthomas/chroma/v2/styles"
 	"github.com/google/go-cmp/cmp"
 	"github.com/walles/moor/v2/internal/linemetadata"
@@ -100,19 +98,28 @@ func TestBrokenUtf8(t *testing.T) {
 
 func startPaging(t *testing.T, reader *reader.ReaderImpl) *twin.FakeScreen {
 	// 0 means default tab size. Defaults to 8 to be like less.
-	return startPagingWithTabSize(t, 0, reader)
+	return startPagingWithTabSizeAndScreen(t, 0, twin.NewFakeScreen(20, 10), reader)
 }
 
 func startPagingWithTabSize(t *testing.T, tabSize int, reader *reader.ReaderImpl) *twin.FakeScreen {
+	return startPagingWithTabSizeAndScreen(t, tabSize, twin.NewFakeScreen(20, 10), reader)
+}
+
+func startPagingWithScreen(t *testing.T, screen *twin.FakeScreen, reader *reader.ReaderImpl) *twin.FakeScreen {
+	// 0 means default tab size. Defaults to 8 to be like less.
+	return startPagingWithTabSizeAndScreen(t, 0, screen, reader)
+}
+
+func startPagingWithTabSizeAndScreen(t *testing.T, tabSize int, screen *twin.FakeScreen, reader *reader.ReaderImpl) *twin.FakeScreen {
 	err := reader.Wait()
 	if err != nil {
 		t.Fatalf("Failed waiting for reader: %v", err)
 	}
 
-	screen := twin.NewFakeScreen(20, 10)
 	pager := NewPager(reader)
 	pager.TabSize = tabSize
 	pager.ShowLineNumbers = false
+	pager.showLineNumbers = false
 
 	// Tell our Pager to quit immediately
 	pager.Quit()
@@ -136,6 +143,7 @@ func startPagingWithTerminalFg(t *testing.T, reader *reader.ReaderImpl, withTerm
 	screen := twin.NewFakeScreen(20, 10)
 	pager := NewPager(reader)
 	pager.ShowLineNumbers = false
+	pager.showLineNumbers = false
 	pager.WithTerminalFg = withTerminalFg
 
 	// Tell our Pager to quit immediately
@@ -329,91 +337,6 @@ func TestManPageFormatting(t *testing.T) {
 	// FIXME: Test backspace between two uncombinable characters
 }
 
-func TestToPattern(t *testing.T) {
-	assert.Assert(t, toPattern("") == nil)
-
-	// Test regexp matching
-	assert.Assert(t, toPattern("G.*S").MatchString("GRIIIS"))
-	assert.Assert(t, !toPattern("G.*S").MatchString("gRIIIS"))
-
-	// Test case insensitive regexp matching
-	assert.Assert(t, toPattern("g.*s").MatchString("GRIIIS"))
-	assert.Assert(t, toPattern("g.*s").MatchString("gRIIIS"))
-
-	// Test non-regexp matching
-	assert.Assert(t, toPattern(")G").MatchString(")G"))
-	assert.Assert(t, !toPattern(")G").MatchString(")g"))
-
-	// Test case insensitive non-regexp matching
-	assert.Assert(t, toPattern(")g").MatchString(")G"))
-	assert.Assert(t, toPattern(")g").MatchString(")g"))
-}
-
-func TestFindFirstHitSimple(t *testing.T) {
-	reader := reader.NewFromTextForTesting("TestFindFirstHitSimple", "AB")
-	pager := NewPager(reader)
-	pager.screen = twin.NewFakeScreen(40, 10)
-
-	assert.NilError(t, pager.readers[pager.currentReader].Wait())
-
-	pager.searchPattern = toPattern("AB")
-
-	hit := pager.findFirstHit(linemetadata.Index{}, nil, false)
-	assert.Assert(t, hit.internalDontTouch.lineIndex.IsZero())
-	assert.Equal(t, hit.internalDontTouch.deltaScreenLines, 0)
-}
-
-func TestFindFirstHitAnsi(t *testing.T) {
-	reader := reader.NewFromTextForTesting("", "A\x1b[30mB")
-	pager := NewPager(reader)
-	pager.screen = twin.NewFakeScreen(40, 10)
-
-	assert.NilError(t, pager.readers[pager.currentReader].Wait())
-
-	pager.searchPattern = toPattern("AB")
-
-	hit := pager.findFirstHit(linemetadata.Index{}, nil, false)
-	assert.Assert(t, hit.internalDontTouch.lineIndex.IsZero())
-	assert.Equal(t, hit.internalDontTouch.deltaScreenLines, 0)
-}
-
-func TestFindFirstHitNoMatch(t *testing.T) {
-	reader := reader.NewFromTextForTesting("TestFindFirstHitSimple", "AB")
-	pager := NewPager(reader)
-	pager.screen = twin.NewFakeScreen(40, 10)
-
-	assert.NilError(t, pager.readers[pager.currentReader].Wait())
-
-	pager.searchPattern = toPattern("this pattern should not be found")
-
-	hit := pager.findFirstHit(linemetadata.Index{}, nil, false)
-	assert.Assert(t, hit == nil)
-}
-
-func TestFindFirstHitNoMatchBackwards(t *testing.T) {
-	reader := reader.NewFromTextForTesting("TestFindFirstHitSimple", "AB")
-	pager := NewPager(reader)
-	pager.screen = twin.NewFakeScreen(40, 10)
-
-	assert.NilError(t, pager.readers[pager.currentReader].Wait())
-
-	pager.searchPattern = toPattern("this pattern should not be found")
-	theEnd := *linemetadata.IndexFromLength(reader.GetLineCount())
-
-	hit := pager.findFirstHit(theEnd, nil, true)
-	assert.Assert(t, hit == nil)
-}
-
-// Converts a cell row to a plain string and removes trailing whitespace.
-func rowToString(row []twin.StyledRune) string {
-	rowString := ""
-	for _, cell := range row {
-		rowString += string(cell.Rune)
-	}
-
-	return strings.TrimRight(rowString, " ")
-}
-
 func TestScrollToBottomWrapNextToLastLine(t *testing.T) {
 	reader := reader.NewFromTextForTesting("",
 		"first line\nline two will be wrapped\nhere's the last line")
@@ -424,6 +347,7 @@ func TestScrollToBottomWrapNextToLastLine(t *testing.T) {
 	pager := NewPager(reader)
 	pager.WrapLongLines = true
 	pager.ShowLineNumbers = false
+	pager.showLineNumbers = false
 	pager.screen = screen
 
 	assert.NilError(t, pager.readers[pager.currentReader].Wait())
@@ -459,6 +383,7 @@ func TestScrollToEndLongInput(t *testing.T) {
 	reader := reader.NewFromTextForTesting("test", strings.Repeat(".\n", lineCount-1)+"X")
 	pager := NewPager(reader)
 	pager.ShowLineNumbers = true
+	pager.showLineNumbers = true
 
 	// Tell our Pager to quit immediately
 	pager.Quit()
@@ -606,6 +531,7 @@ func TestPageSamples(t *testing.T) {
 			pager := NewPager(myReader)
 			pager.WrapLongLines = false
 			pager.ShowLineNumbers = false
+			pager.showLineNumbers = false
 
 			// Heigh 3 = two lines of contents + one footer
 			screen := twin.NewFakeScreen(10, 3)
@@ -673,6 +599,7 @@ func TestClearToEndOfLine_ClearFromNotStart(t *testing.T) {
 func TestClearToEndOfLine_ClearFromStartScrolledRight(t *testing.T) {
 	pager := NewPager(reader.NewFromTextForTesting("TestClearToEol", blueBackgroundClearToEol0))
 	pager.ShowLineNumbers = false
+	pager.showLineNumbers = false
 
 	// Tell our Pager to quit immediately
 	pager.Quit()
@@ -736,68 +663,20 @@ func TestTerminalFg(t *testing.T) {
 	assertRunesEqual(t, terminalAnswer, startPagingWithTerminalFg(t, reader, true).GetRow(0)[0])
 }
 
-func benchmarkSearch(b *testing.B, highlighted bool) {
-	// Pick a go file so we get something with highlighting
-	_, sourceFilename, _, ok := runtime.Caller(0)
-	if !ok {
-		panic("Getting current filename failed")
-	}
-
-	sourceBytes, err := os.ReadFile(sourceFilename)
-	assert.NilError(b, err)
-	fileContents := string(sourceBytes)
-
-	// Read one copy of the example input
-	if highlighted {
-		highlightedSourceCode, err := reader.Highlight(fileContents, *styles.Get("native"), formatters.TTY16m, lexers.Get("go"))
-		assert.NilError(b, err)
-		if highlightedSourceCode == nil {
-			panic("Highlighting didn't want to, returned nil")
-		}
-		fileContents = *highlightedSourceCode
-	}
-
-	// Create some input to search
-	testString := ""
-	for range 100 {
-		testString += fileContents
-	}
-
-	reader := reader.NewFromTextForTesting("hello", testString)
-	pager := NewPager(reader)
-	pager.screen = twin.NewFakeScreen(40, 10)
-
-	// The [] around the 't' is there to make sure it doesn't match, remember
-	// we're searching through this very file.
-	pager.searchPattern = regexp.MustCompile("This won'[t] match anything")
-
-	// I hope forcing a GC here will make numbers more predictable
-	runtime.GC()
-
-	b.ResetTimer()
-
-	for range b.N {
-		// This test will search through all the N copies we made of our file
-		hit := pager.findFirstHit(linemetadata.Index{}, nil, false)
-
-		if hit != nil {
-			panic(fmt.Errorf("This test is meant to scan the whole file without finding anything"))
-		}
-	}
+func testFooter(t *testing.T, filename string, contents string, expectedFooter string) {
+	reader := reader.NewFromTextForTesting(filename, contents)
+	screen := startPagingWithScreen(t, twin.NewFakeScreen(999, 10), reader)
+	footer := rowToString(screen.GetRow(9))
+	assert.Equal(t, expectedFooter, footer, fmt.Sprintf("filename='%s', contents='%s'", filename, contents))
 }
 
-// How long does it take to search a highlighted file for some regex?
-//
-// Run with: go test -run='^$' -bench=. . ./...
-func BenchmarkHighlightedSearch(b *testing.B) {
-	benchmarkSearch(b, true)
-}
+func TestFooter(t *testing.T) {
+	help := "Press ESC / q to exit, / to search, & to filter, h for help"
 
-// How long does it take to search a plain text file for some regex?
-//
-// Search performance was a problem for me when I had a 600MB file to search in.
-//
-// Run with: go test -run='^$' -bench=. . ./...
-func BenchmarkPlainTextSearch(b *testing.B) {
-	benchmarkSearch(b, false)
+	testFooter(t, "filename", "", "filename: <empty>  "+help)
+	testFooter(t, "", "", "<empty>  "+help)
+	testFooter(t, "", "text", "1 line  100%  "+help)
+	testFooter(t, "filename", "text", "filename: 1 line  100%  "+help)
+
+	testFooter(t, "", "line 1\nline 2", "2 lines  100%  "+help)
 }
